package TiVo::Zap2it::Parser;

use strict;
use base qw(TiVo::Web::Parser);
use HTML::TokeParser;
use Time::Local;
use TiVo::ServerId::Series;
use TiVo::ServerId::Program;
use TiVo::Show;

our $VERSION =
    (split / /, q$Id: Parser.pm,v 1.12 2002/03/19 18:15:44 grant Exp $)[2];

{
    my %genres = (
                  # zapid, tivoid
                  1 => 1,           # action / adventure
                  2 => 2,           # adult
                  3 => 68,          # movie
                  4 => undef,       # series
                  5 => 16,          # biography
                  6 => 21,          # business / financial
                  7 => 22,          # children
                  8 => 25,          # comedy
                  9 => 29,          # crime
                  10 => 34,         # documentary
                  11 => 35,         # drama
                  12 => 36,         # educational
                  
                  14 => 48,         # health / fitness
                  15 => 55,         # horror
                  
                  17 => 70,         # musical
                  18 => 71,         # mystery
                  19 => 113,        # variety
                  20 => 73,         # news
                  21 => 82,         # romance
                  22 => 88,         # science fiction
                  23 => 90,         # shopping
                  24 => 96,         # soap opera
                  25 => 100,        # special
                  26 => 80,         # spiritual / religion
                  27 => 101,        # sports
                  28 => 101,        # sport events
                  29 => 102,        # sports info
                  
                  31 => 106,        # talk / tabloid
                  32 => 118,        # western
                  33 => 3,          # animals
                  
                  56 => undef,      # SPORTS NEWS (wtf?)
                  
                  74 => undef,      # entertainment news
                  
                  75 => 79,         # reality

                  77 => 84,         # rugby
                  
                  79 => 26,         # computers
                  80 => 27,         # cooking
                  81 => 32,         # dance
                  82 => 37,         # electronics
                  83 => 40,         # fashion
                  84 => 43,         # French
                  85 => 48,         # health / fitness
                  86 => 49,         # historical
                  87 => 54,         # home and garden
                  88 => 57,         # housewares
                  89 => 58,         # howto
                  91 => 61,         # jewelry
                  92 => 65,         # medical
                  93 => 72,         # nature
                  
                  97 => 112,        # travel
                  
                  99 => undef,      # info
                  100 => 91,        # situation
                  
                  102 => 115,       # war
                  103 => 44,        # fundraiser
                  104 => 24,        # collectibles
                  
                  106 => 6,         # art
                  107 => 13,        # beauty
                  108 => 45,        # game show
                  109 => 23,        # classic
                 );
    
    # returns 0 if not found
    # if the genre is there, it can still be undef (no TiVO analogue)
    sub genre {
        my $self = shift;
        my $zapgenre = shift;
        return (exists $genres{$zapgenre}) ? $genres{$zapgenre} : 0;
    }
}

# takes a page object
# returns a list of TiVo::Show objects
sub parse_page {
    my $self = shift;
    my $page = shift;

    my @shows;
    my $content = $page->content;
    my $p = HTML::TokeParser->new(\$content) 
        or die "failed to create parser";

    until ($p->get_tag('select')->[1]{name} eq 'startDay') {} # twiddle thumbs
                                                   
    while (my $block = $p->get_tag('tr')) {
        my $show = TiVo::Show->new;
        $show->station($page->station);
        while (my $token = $p->get_tag('a', 'font', 'b', '/tr')) {
            if ($token->[0] eq 'a') {   # pid,sid,title,episode
                my $url = $token->[1]{href};
                if ($url =~ /^progdetails/) {
                    my ($pid) = $url =~ /prog_id=(\d+)/;
                    my ($sid) = $url =~ /series_id=(\d+)/;
                    my $text = $p->get_trimmed_text("/a");
                    if ($sid) {
                        $show->series($sid);
                        $show->episode($text);
                    }
                    else {
                        $show->program($pid);
                        $text =~ s/Part \d+ of \d+//;
                        $show->title($text);
                    }
                }
                elsif ($url =~ /^textone.*cat_id=(\d+)/) {
                    my $catid = $1;
                    my $genre = $self->genre($catid);
                    if ($genre) {
                        $show->genre($genre);
                    }
                    elsif (not defined $genre) {
                        # ignore
                    }
                    else {
                        my $text = $p->get_trimmed_text('/a');
                        warn "no genre found for $catid $text\n";
                    }
                }
            }
            elsif ($token->[0] eq 'font') { # description, end
                my $size = $token->[1]{size};
                next unless $size == -2;
                my $text = $p->get_trimmed_text();
                my ($desc, $end) = split /\(ends at /, $text;
                $end =~ s/\)\s*$//;
                $show->description($desc) if $desc; # sometimes blank
                $show->end($end);
            }
            elsif ($token->[0] eq 'b') { # time
                my $text = $p->get_trimmed_text();
                next unless $text =~ /AM|PM/;
                $show->start($text);
            }
            elsif ($token->[0] eq '/tr') {
                last;               # bust this loop
            }
        }

        next unless $show->title; # every show should have a title
                                                   
        # make (hopefully) unique series and program ids
        if ($show->series) {
            my $id = TiVo::ServerId::Series->new($show->series);
            $show->series($id->value);
            $show->is_episode(1);
        }
        else {
            my $id = TiVo::ServerId::Series->new(
                                                 $show->program, 
                                                 from_program => 1,
                                                );
            $show->series($id->value);
            $show->is_episode(0);
        }
        $show->episodic(1); # FIXME! should be 0 for movies
        my $id = TiVo::ServerId::Program->new($show->program);
        $show->program($id->value);
                                                   
        # change time to GMT days and offset
        # FIXME this is nasty shit
        my ($starttime, $ampm) = split / /, $show->start;
        my ($starthour, $startmin) = split /:/, $starttime;
        my ($endhour, $endmin) = split /:/, $show->end;
        $endmin =~ s/\D*$//;
                             
        # 12:00 AM -> 0000
        if ($starthour == 12 and $ampm eq 'AM') {
            $starthour = 0;
            $endhour = 0 if $endhour == 12; # eg, ends at 12:30
        }

        # change to 24-hour
        if ($ampm eq 'PM' and $starthour < 12) {
            $starthour += 12;
            $endhour += 12;     # this will go past 23, but that's okay
        }

        $endhour += 12 if $endhour < $starthour;

        my $startint = $starthour * 60 + $startmin;
        my $endint = $endhour * 60 + $endmin;
        $show->duration(($endint - $startint) * 60);

        my ($year, $month, $day) = ($page->day =~ /(\d{4})(\d{2})(\d{2})/);
        $month--;
        my $time = timelocal(0, $startmin, $starthour, $day, $month, $year);
        $show->day(int $time / 86400);
        $show->time($time % 86400);

        # FIXME? could remove 'start' and 'end'

        push @shows, $show;
    }
    return @shows;
}

1;
